# Copyright (C) 2020 by ÿnérant, eichhornchen, nicomarg, charlse
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from random import randint
from typing import List

from squirrelbattle.interfaces import Map, Tile
from squirrelbattle.mapgeneration import broguelike


class TestBroguelike(unittest.TestCase):
    def setUp(self) -> None:
        self.generator = broguelike.Generator()

    def is_connex(self, grid: List[List[Tile]]) -> bool:
        h, w = len(grid), len(grid[0])
        y, x = randint(0, h - 1), randint(0, w - 1)
        while not (grid[y][x].can_walk()):
            y, x = randint(0, h - 1), randint(0, w - 1)
        queue = Map.neighbourhood(grid, y, x)
        while queue:
            y, x = queue.pop()
            if grid[y][x].can_walk():
                grid[y][x] = Tile.WALL
                queue += Map.neighbourhood(grid, y, x)
        return not any([t.can_walk() for row in grid for t in row])

    def test_connexity(self) -> None:
        m = self.generator.run()
        self.assertTrue(self.is_connex(m.tiles))
